from typing import Dict

from django.contrib.auth import get_user_model


def get_user_by_email(userinfo_token: Dict[str, str], id_token_claims: Dict):
    User = get_user_model()

    username = ""

    if "preferred_username" in id_token_claims:
        username = id_token_claims["preferred_username"]
    elif "preferred_username" in userinfo_token:
        username = userinfo_token["preferred_username"]
    else:
        username = userinfo_token["email"]

    user, created = User.objects.get_or_create(
        email=userinfo_token["email"],
        username=username,
    )
    user.backend = "django.contrib.auth.backends.ModelBackend"
    return user



import logging

from django.test import TestCase, override_settings

logger = logging.getLogger(__name__)


@override_settings(
    DJANGO_PYOIDC={
        "sso1": {
            "OIDC_CACHE_PROVIDER_METADATA": False,
            "OIDC_CLIENT_ID": "1",
            "CACHE_DJANGO_BACKEND": "default",
            "OIDC_PROVIDER_DISCOVERY_URI": "",
            "OIDC_CLIENT_SECRET": "",
            "OIDC_CALLBACK_PATH": "/callback",
            "POST_LOGIN_URI_SUCCESS": "/default/success",
            "LOGIN_URIS_REDIRECT_ALLOWED_HOSTS": ["test.django-pyoidc.notatld"],
            "LOGIN_REDIRECTION_REQUIRES_HTTPS": True,
            "POST_LOGOUT_REDIRECT_URI": "/logoutdone",
            "POST_LOGIN_URI_FAILURE": "/logout_failure",
        },
        "sso2": {
            "OIDC_CACHE_PROVIDER_METADATA": False,
            "OIDC_CLIENT_ID": "2",
            "CACHE_DJANGO_BACKEND": "default",
            "OIDC_PROVIDER_DISCOVERY_URI": "",
            "OIDC_CLIENT_SECRET": "",
        },
        "sso3": {
            "OIDC_CACHE_PROVIDER_METADATA": True,
            "OIDC_CLIENT_ID": "3",
            "CACHE_DJANGO_BACKEND": "default",
            "OIDC_PROVIDER_DISCOVERY_URI": "http://sso3/uri",
            "OIDC_CLIENT_SECRET": "",
        },
        "sso4": {
            "OIDC_CACHE_PROVIDER_METADATA": True,
            "OIDC_CLIENT_ID": "4",
            "CACHE_DJANGO_BACKEND": "default",
            "OIDC_PROVIDER_DISCOVERY_URI": "http://sso4/uri",
            "OIDC_CLIENT_SECRET": "",
        },
    }
)
class OIDCTestCase(TestCase):
    pass



# Generated by Django 3.2.16 on 2023-04-04 05:35

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="OIDCSession",
            fields=[
                ("id", models.BigAutoField(primary_key=True, serialize=False)),
                ("state", models.TextField()),
                ("session_state", models.TextField(null=True)),
                ("sub", models.TextField()),
                ("cache_session_key", models.TextField()),
                ("created_at", models.DateTimeField(auto_now_add=True)),
            ],
        )
    ]



from typing import Any, Dict, List

from django.urls import path, reverse_lazy


class Provider:
    """
    This is the base `Provider` class that is used to implement common provider configuration patterns. You should not
    use this class directly. Instead, you should but subclass it to implement the configuration logic.
    """

    def __init__(
        self,
        op_name: str,
        provider_discovery_uri: str,
        logout_redirect: str,
        failure_redirect: str,
        success_redirect: str,
        redirect_requires_https: bool,
        client_secret: str,
        client_id: str,
    ):
        """
        Parameters:
            op_name (str): the name of the sso provider that you are using
            logout_redirect (str): the URI where a user should be redirected to on logout success
            failure_redirect (str): the URI where a user should be redirected to on login failure
            success_redirect (str): the URI a user should be redirected to on login success if no redirection url where provided
            redirect_requires_https (bool): set to True to disallow redirecting user to non-https uri on login success
            client_secret (str): the OIDC client secret
            client_id (str): the OIDC client ID
        """
        self.op_name = op_name
        self.provider_discovery_uri = provider_discovery_uri
        self.logout_redirect = logout_redirect
        self.failure_redirect = failure_redirect
        self.success_redirect = success_redirect
        self.redirect_requires_https = redirect_requires_https
        self.client_secret = client_secret
        self.client_id = client_id

    def get_config(
        self, allowed_hosts, cache_backend: str = "default"
    ) -> Dict[str, Dict[str, Any]]:
        """
        Parameters:
            allowed_hosts(:obj:`list`) : A list of allowed domains that can be redirected to. A good idea is to this to
             :setting:`ALLOWED_HOSTS <django:ALLOWED_HOSTS>`. See :ref:`Redirect the user after login` for more details.
            cache_backend(:obj:`str`, optional): Defaults to 'default'. The cache backend that should be used to store
                this provider sessions. Take a look at :ref:`Cache Management`

        Returns:
            dict: A dictionary with all the settings that `django-pyoidc` expects to work properly
        """
        return {
            self.op_name: {
                "POST_LOGIN_URI_FAILURE": self.failure_redirect,
                "POST_LOGIN_URI_SUCCESS": self.success_redirect,
                "POST_LOGOUT_REDIRECT_URI": self.logout_redirect,
                "OIDC_CALLBACK_PATH": reverse_lazy(self.callback_uri_name),
                "REDIRECT_REQUIRES_HTTPS": self.redirect_requires_https,
                "LOGIN_URIS_REDIRECT_ALLOWED_HOSTS": allowed_hosts,
                "OIDC_CLIENT_SECRET": self.client_secret,
                "OIDC_CLIENT_ID": self.client_id,
                "OIDC_PROVIDER_DISCOVERY_URI": self.provider_discovery_uri,
                "OIDC_LOGOUT_REDIRECT_PARAMETER_NAME": None,
                "CACHE_DJANGO_BACKEND": cache_backend,
            }
        }

    @property
    def login_uri_name(self):
        """
        The login viewname (use in :func:`django:django.urls.reverse` django directive for example) of this configuration
        """
        return f"{self.op_name}-login"

    @property
    def logout_uri_name(self):
        """
        The logout viewname (use in :func:`django:django.urls.reverse` django directive for example) of this configuration
        """
        return f"{self.op_name}-logout"

    @property
    def callback_uri_name(self):
        """
        The callback viewname (use in :func:`django:django.urls.reverse` django directive for example) of this configuration
        """
        return f"{self.op_name}-callback"

    def get_urlpatterns(self) -> List[Any]:
        """
        Returns:
            A list of urllpatterns to be included using :func:`django:django.urls.include` in your urllconfiguration
        """
        from django_pyoidc.views import OIDCCallbackView, OIDCLoginView, OIDCLogoutView

        result = [
            path(
                "callback",
                OIDCCallbackView.as_view(op_name=self.op_name),
                name=self.callback_uri_name,
            ),
            path(
                "login",
                OIDCLoginView.as_view(op_name=self.op_name),
                name=self.login_uri_name,
            ),
            path(
                "logout",
                OIDCLogoutView.as_view(op_name=self.op_name),
                name=self.logout_uri_name,
            ),
        ]
        return result



import hashlib
import logging
from typing import Dict, Union

from django.conf import settings
from django.core.cache import BaseCache, caches

logger = logging.getLogger(__name__)


def get_setting_for_sso_op(op_name: str, key: str, default=None):
    if key in settings.DJANGO_PYOIDC[op_name]:
        return settings.DJANGO_PYOIDC[op_name][key]
    else:
        return default


def get_settings_for_sso_op(op_name: str):
    return settings.DJANGO_PYOIDC[op_name]


class OIDCCacheBackendForDjango:
    """Implement General cache for OIDC using django cache"""

    def __init__(self, op_name):
        self.op_name = op_name
        self.enabled = get_setting_for_sso_op(
            self.op_name, "OIDC_CACHE_PROVIDER_METADATA", False
        )
        if self.enabled:
            self.storage: BaseCache = caches[
                get_setting_for_sso_op(self.op_name, "CACHE_DJANGO_BACKEND")
            ]

    def generate_hashed_cache_key(self, value: str) -> str:
        h = hashlib.new("sha256")
        h.update(value.encode())
        cache_key = h.hexdigest()
        return cache_key

    def clear(self):
        return self.storage.clear()

    def get_key(self, key):
        return f"oidc-{self.op_name}-{key}"

    def set(self, key: str, value: Dict[str, Union[str, bool]], expiry: int) -> None:
        if self.enabled:
            self.storage.set(self.get_key(key), value, expiry)

    def __getitem__(self, key: str) -> Dict[str, Union[str, bool]]:
        if self.enabled:
            data = self.storage.get(self.get_key(key))
            if data is None:
                raise KeyError  # Makes __getItem__ handle like Python dict
            return data
        else:
            raise KeyError



from unittest import mock
from unittest.mock import call

from django_pyoidc.utils import OIDCCacheBackendForDjango
from django_pyoidc.views import OIDClient
from tests.utils import OIDCTestCase


class SessionTestCase(OIDCTestCase):
    @mock.patch("django_pyoidc.views.Consumer.provider_config")
    def test_session_isolation_between_providers(self, mocked_provider_config):
        """
        Test that different SSO providers using same SID do not conflict
        """
        sso1 = OIDClient(op_name="sso1")
        sso2 = OIDClient(op_name="sso2")

        mocked_provider_config.assert_has_calls([call(""), call("")])
        assert 2 == mocked_provider_config.call_count

        sso1.consumer._backup(sid="1234")
        sso2.consumer._backup(sid="1234")

        # no more calls
        mocked_provider_config.assert_has_calls([call(""), call("")])
        assert 2 == mocked_provider_config.call_count

        client1 = OIDClient(op_name="sso1", session_id="1234")
        self.assertEqual(client1.consumer.client_id, "1")

        client2 = OIDClient(op_name="sso2", session_id="1234")

        # no more calls
        mocked_provider_config.assert_has_calls([call(""), call("")])
        assert 2 == mocked_provider_config.call_count

        self.assertEqual(client2.consumer.client_id, "2")

    @mock.patch(
        "django_pyoidc.views.Consumer.provider_config",
        return_value=('[{"foo": "bar"}]'),
    )
    def test_session_isolation_between_providers_cached(self, mocked_provider_config):
        """
        Test that different SSO providers with active cache using same SID do not conflict
        """

        # empty the caches
        cache1 = OIDCCacheBackendForDjango("sso3")
        cache2 = OIDCCacheBackendForDjango("sso4")
        cache1.clear()
        cache2.clear()

        sso1 = OIDClient(op_name="sso3")
        mocked_provider_config.assert_has_calls([call("http://sso3/uri")])
        assert 1 == mocked_provider_config.call_count

        sso2 = OIDClient(op_name="sso4")
        mocked_provider_config.assert_has_calls(
            [call("http://sso3/uri"), call("http://sso4/uri")]
        )
        assert 2 == mocked_provider_config.call_count

        sso1.consumer._backup(sid="1234")
        sso2.consumer._backup(sid="1234")

        client1 = OIDClient(op_name="sso3", session_id="1234")
        self.assertEqual(client1.consumer.client_id, "3")

        client2 = OIDClient(op_name="sso4", session_id="1234")
        self.assertEqual(client2.consumer.client_id, "4")

        mocked_provider_config.assert_has_calls(
            [call("http://sso3/uri"), call("http://sso4/uri")]
        )
        assert 2 == mocked_provider_config.call_count



from typing import Any, Dict

from django_pyoidc.providers.keycloak_10 import Keycloak10Provider


class Keycloak17Provider(Keycloak10Provider):
    """
    Provide Django settings/urlconf based on keycloak behaviour (v17)
    """


import os

from decouple import config

PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

SECRET_KEY = "fake-key"
INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.sessions",
    "tests",
    "django_pyoidc",
]

ALLOWED_HOSTS = ["test.django-pyoidc.notatld"]

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(PROJECT_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

# SESSION_ENGINE = "django.contrib.sessions.backends.file"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "HOST": config("POSTGRES_HOST", "db"),
        "USER": config("POSTGRES_USER", "postgres"),
        "NAME": config("POSTGRES_DB", "postgres"),
        "PASSWORD": config("POSTGRES_PASSWORD", "postgres"),
        "PORT": config("POSTGRES_PORT", default=5432),
    }
}


ROOT_URLCONF = "tests.urls"

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
    }
}



from django_pyoidc.providers.lemonldapng2 import LemonLDAPng2Provider


class LemonLDAPngProvider(LemonLDAPng2Provider):
    """
    Provide django settings/urlconf based on LemonLDAP-ng behaviour (latest version).

    For older LemonLDAPNg versions please check the other Klemonldapng_* providers.
    """

    pass



from typing import Any, Dict

from django_pyoidc.providers.base import Provider


class LemonLDAPng2Provider(Provider):
    """
    Provide Django settings/urlconf based on LemonLDAP-ng behaviour (v2)
    """

    def get_config(self, allowed_hosts) -> Dict[str, Dict[str, Any]]:
        result = super().get_config(allowed_hosts)
        # logout is by default asking for confirmation unless you pass confirm=1
        result[self.op_name]["LOGOUT_QUERY_STRING_EXTRA_PARAMETERS_DICT"] = {
            "confirm": 1
        }
        return result



from typing import Any, Dict

from django_pyoidc.providers.keycloak_17 import Keycloak17Provider


class Keycloak18Provider(Keycloak17Provider):
    """
    Provide Django settings/urlconf based on keycloak behaviour (v18)
    """

    def get_config(self, allowed_hosts) -> Dict[str, Dict[str, Any]]:
        result = super().get_config(allowed_hosts)
        # logout redirection query string parameter name altered, fromredirect_uri to post_logout_redirect_uri
        result[self.op_name][
            "OIDC_LOGOUT_QUERY_STRING_REDIRECT_PARAMETER"
        ] = "post_logout_redirect_uri"
        return result



from django.db import models


class OIDCSession(models.Model):
    id = models.BigAutoField(primary_key=True)

    # Used by pyoidc to save the client when no session state is available
    state = models.TextField()

    # Defined in https://openid.net/specs/openid-connect-session-1_0.html, use to find back a session using 'sid' from a logout JWT
    session_state = models.TextField(null=True)

    # Used to find back a session using 'sub' from a logout JWT
    sub = models.TextField()

    # Django cache session key used to log out users by finding a session in a SessionStore and deleting it
    cache_session_key = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        if self.session_state:
            return f"Session with id : {self.session_state}"
        else:
            return f"Session without id, sub : {self.sub}"



"""
toto
"""
from typing import Any, Dict

from django_pyoidc.providers.base import Provider


class Keycloak10Provider(Provider):
    """
    Provide Django settings/urlconf based on keycloak behaviour (v10 to v18)
    """

    def __init__(self, keycloak_base_uri: str, keycloak_realm: str, *args, **kwargs):
        self.keycloak_base_uri = keycloak_base_uri
        self.keycloak_realm = keycloak_realm
        provider_discovery_uri = f"{keycloak_base_uri}/{keycloak_realm}"
        super().__init__(*args, **kwargs, provider_discovery_uri=provider_discovery_uri)

    def get_config(self, allowed_hosts, **kwargs) -> Dict[str, Dict[str, Any]]:
        result = super().get_config(allowed_hosts, **kwargs)
        # result[self.op_name]["SCOPE"] = "full-dedicated"
        result[self.op_name][
            "OIDC_LOGOUT_QUERY_STRING_REDIRECT_PARAMETER"
        ] = "redirect_uri"
        return result



#!/usr/bin/env python


import os

from setuptools import find_packages, setup

HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "django_pyoidc", "VERSION")) as version_file:
    VERSION = version_file.read().strip()

setup(
    version=VERSION,
    packages=find_packages(),
)



import base64
import logging
from typing import Dict, List, Union

import jsonpickle
from Cryptodome.PublicKey.RSA import RsaKey, import_key
from django.core.cache import BaseCache, caches
from jsonpickle.handlers import BaseHandler
from oic.utils.session_backend import SessionBackend

from django_pyoidc.models import OIDCSession
from django_pyoidc.utils import get_settings_for_sso_op

logger = logging.getLogger(__name__)


# From https://github.com/alehuo/pyoidc-redis-session-backend/blob/master/pyoidc_redis_session_backend/__init__.py
class RSAKeyHandler(BaseHandler):
    def flatten(self, obj: RsaKey, data):
        data["rsa_key"] = base64.b64encode(obj.export_key()).decode("utf-8")
        return data

    def restore(self, obj):
        return import_key(base64.b64decode(obj["rsa_key"]))


jsonpickle.register(RsaKey, RSAKeyHandler)


class OIDCCacheSessionBackendForDjango(SessionBackend):
    """Implement Session backend using django cache"""

    def __init__(self, op_name):
        self.storage: BaseCache = caches[
            get_settings_for_sso_op(op_name)["CACHE_DJANGO_BACKEND"]
        ]
        self.op_name = op_name

    def get_key(self, key):
        return f"{self.op_name}-{key}"

    def __setitem__(self, key: str, value: Dict[str, Union[str, bool]]) -> None:
        self.storage.set(self.get_key(key), jsonpickle.encode(value))

    def __getitem__(self, key: str) -> Dict[str, Union[str, bool]]:
        data = self.storage.get(self.get_key(key))
        if data is None:
            raise KeyError  # Makes __getItem__ handle like Python dict
        return jsonpickle.decode(data)

    def __delitem__(self, key: str) -> None:
        self.storage.delete(self.get_key(key))

    def __contains__(self, key: str) -> bool:
        return self.storage.get(self.get_key(key)) is not None

    def get_by_uid(self, uid: str) -> List[str]:
        result = OIDCSession.objects.filter(uid=uid).values_list("sid", flat=True)
        logger.debug(f"Fetched the following sid : {result} for {uid=}")

        return result

    def get_by_sub(self, sub: str) -> List[str]:
        result = OIDCSession.objects.filter(sub=sub).values_list("sid", flat=True)
        logger.debug(f"Fetched fhe following sid : {result} for {sub=}")
        return result

    def get(self, attr: str, val: str) -> List[str]:
        logger.debug(f"Fetch SID for sessions where [{attr}] = {val}")
        raise NotImplementedError(
            "Current session implementation does not support this method"
        )
